﻿using FLY.IBC.Common;
using FLY.IBC.IService;
using FLY.IBC.Server.Model;
using FLY.Modbus;
using FLY.OBJComponents.Common;
using FLY.OBJComponents.IService;
using FLY.OBJComponents.Server;
using FObjBase;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace FLY.IBC.Server
{
    [JsonObject(MemberSerialization.OptIn)]
    public class IBCSystem : IIbcSystemService
    {
        #region IIBCSystem
        /// <summary>
        /// 数据
        /// </summary>
        public IBCData Item { get; } = new IBCData();

        private int ctrlInterval = 5;
        /// <summary>
        /// 控制记录周期,单位s, 最小值为2
        /// </summary>
        [JsonProperty]
        public int CtrlInterval
        {
            get { return ctrlInterval; }
            set
            {
                if (value < 2)
                    value = 2;
                if (ctrlInterval != value)
                {
                    ctrlInterval = value;
                }
            }
        }

        private PLCProxySystem plcos = new PLCProxySystem();
        /// <summary>
        /// PLC代理系统
        /// </summary>
        public IPLCProxySystemService PLCos { get { return plcos; } }

        #endregion


        /// <summary>
        /// 报警系统
        /// </summary>
        WarningSystem warning;
        /// <summary>
        /// 记录到数据库
        /// </summary>
        HistoryDb historyDb;
        /// <summary>
        /// 报警配置
        /// </summary>
        ErrorConf errorConf;
        /// <summary>
        /// 周期保存Ibc数据
        /// </summary>
        PeriodicallySaveData<Db_Width> psdWidth;

        public IBCSystem()
        {
            Load();


        }
        public void Init(HistoryDb historyDb ,WarningSystem warning)
        {
            this.historyDb = historyDb;
            this.warning = warning;

            //--------------------------------------------------------------------------------
            //step 1 加载PLC寄存器 文件
            plcos.AddConfigFile(
                "plcgroup.json",
                (plcgroup) => new Dictionary<string, INotifyPropertyChanged>
                {
                    { "Item", Item }
                }
            );

            //--------------------------------------------------------------------------------
            //step 2 报警配置
            errorConf = new ErrorConf(PLCos, this.warning, "IBC");
            errorConf.AddErrorAction(Item);
            errorConf.InitError();

            //--------------------------------------------------------------------------------
            //step 3 历史数据记录

            //添加任务
            PLCos.SetPlan("Item", new string[] {
                        "FilmWidth","InletAirFreq","OutletAirFreq"
                    }, 0);

            psdWidth = new PeriodicallySaveData<Db_Width>();
            psdWidth.Init(
                CtrlInterval,
                this.historyDb.AddWidthRange,
                () => {
                    if (!PLCos.IsConnectedWithPLC)
                        return null;

                    if (Item.InletAirFreq < 2 && Item.OutletAirFreq < 2)//没有工作！！！！
                        return null;

                    //记录数据
                    return new Db_Width()
                    {
                        Time = DateTime.Now,
                        FilmWidth = Math.Round(Item.FilmWidth, 1),
                        InletAirFreq = Math.Round(Item.InletAirFreq, 1),
                        OutletAirFreq = Math.Round(Item.OutletAirFreq, 1)
                    };
                });

            //--------------------------------------------------------------------------------
            //last step  PLC 更新计划服务初始化
            plcos.Init();



        }


        public event PropertyChangedEventHandler PropertyChanged;
        
        void Save()
        {
            string json = JsonConvert.SerializeObject(this, Formatting.Indented);
            File.WriteAllText("ibc.json", json);
        }
        void Load()
        {
            if (!File.Exists("ibc.json"))
                return;
            string json = File.ReadAllText("ibc.json");
            try
            {
                JsonConvert.PopulateObject(json, this);
            }
            catch 
            {
            
            }
        }
    }

}
