﻿using FLY.Thick.Base.UI;
using FLY.Thick.Blowing.Client;
using FLY.Thick.Blowing.Common;
using FLY.Thick.Blowing.IService;
using FObjBase;
using GalaSoft.MvvmLight.Command;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using Unity;

namespace FLY.Thick.Blowing.UI.Fix.Client
{
    /// <summary>
    /// Page_ProfileBlowing.xaml 的交互逻辑
    /// </summary>
    public partial class PgProfileBlowing : Page
    {
        PgProfileBlowingVm viewModel;
        public PgProfileBlowing()
        {
            InitializeComponent();
        }

        [InjectionMethod]
        public void Init(IBlowingFixProfileService profileService)
        {
            viewModel = new PgProfileBlowingVm();
            viewModel.Init(profileService, listview_profile);
            this.DataContext = viewModel;
        }
    }

    public class PgProfileBlowingVm : INotifyPropertyChanged
    {
        public event PropertyChangedEventHandler PropertyChanged;

        #region 参数
        public ObservableCollection<string> ProfileList { get; } = new ObservableCollection<string>();
        public BlowingFixProfileParam Param { get; } = new BlowingFixProfileParam();
        #endregion

        #region Command
        public RelayCommand ReadCmd { get; }
        public RelayCommand ApplyCmd { get; }
        public RelayCommand DelCmd { get; }

        public RelayCommand AhelperCmd { get; }

        public RelayCommand LegendCmd { get; }
        #endregion

        IBlowingFixProfileService profileService;
        ListBox listview_profile;

        public PgProfileBlowingVm()
        {
            ReadCmd = new RelayCommand(Read);
            ApplyCmd = new RelayCommand(Apply);
            DelCmd = new RelayCommand(Del);
            AhelperCmd = new RelayCommand(Ahelper);
            LegendCmd = new RelayCommand(Legend);
        }

        public void Init(IBlowingFixProfileService profileService, ListBox listview_profile)
        {
            this.profileService = profileService;
            this.listview_profile = listview_profile;

            Param.Copy(this.profileService.Param);

            this.profileService.GetList(
                new AsyncCBHandler(
                    delegate (object AsyncState, object retData)
                    {
                        List<string> list = (List<string>)retData;
                        ProfileList.Clear();
                        for (int i = 0; i < list.Count(); i++)
                            ProfileList.Add(list[i]);

                        if (ProfileList.Contains(Param.PName))
                        {
                            this.listview_profile.SelectedItem = ProfileList.First(s => s == Param.PName);
                        }
                    }), null);
            this.profileService.Param.PropertyChanged += Param_PropertyChanged;
        }

        private void Param_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            //TODO
        }


        private void Read()
        {
            if (listview_profile.SelectedItem != null)
            {
                string productname = listview_profile.SelectedItem as string;
                profileService.Read(
                    productname,
                    new AsyncCBHandler(
                    delegate (object AsyncState, object retData)
                    {
                        var p = (BlowingFixProfileParam)retData;
                        Misc.PropertiesManager.CopyTo(p, Param);
                    }), null);
            }
        }

        private void Apply()
        {
            //有效性检查
            switch (Param.MMode)
            {
                case MeasureMode.Edge:
                    break;
                case MeasureMode.Normal:
                    {
                        if (Param.FilmWidth < 500)
                        {
                            FLY.ControlLibrary.Window_WarningTip.Show("参数出错", "膜宽<500mm 异常");
                            return;
                        }
                        else if (Param.FilmPosH > Param.FilmWidth / 3)
                        {
                            FLY.ControlLibrary.Window_WarningTip.Show("参数出错", "探头位置>(膜宽/3) 异常");
                            return;
                        }
                        else if (Param.FilmPosH < 0)
                        {
                            FLY.ControlLibrary.Window_WarningTip.Show("参数出错", "探头位置<0 异常");
                            return;
                        }
                    }
                    break;
                default:
                    //case MeasureMode.Bag:
                    {
                        if (Param.FilmWidth < 500)
                        {
                            FLY.ControlLibrary.Window_WarningTip.Show("参数出错", "膜宽<500mm 异常");
                            return;
                        }
                        else if (Param.FilmPosH > Param.FilmWidth / 3)
                        {
                            FLY.ControlLibrary.Window_WarningTip.Show("参数出错", "探头位置>(膜宽/3) 异常");
                            return;
                        }
                        else if (Param.FilmPosH < 0)
                        {
                            FLY.ControlLibrary.Window_WarningTip.Show("参数出错", "探头位置<0 异常");
                            return;
                        }
                        else if (Param.BagFold0 < Param.FilmPosH)
                        {
                            FLY.ControlLibrary.Window_WarningTip.Show("参数出错", "探头下的折边 < 探头位置");
                            return;
                        }
                        else if (Param.BagFold1 < Param.FilmPosH)
                        {
                            FLY.ControlLibrary.Window_WarningTip.Show("参数出错", "另一边的折边 < 探头位置");
                            return;
                        }
                        else if (Param.FilmWidth < (Param.BagFold0 + Param.BagFold1))
                        {
                            FLY.ControlLibrary.Window_WarningTip.Show("参数出错", "折边 >(膜宽/2) 异常");
                            return;
                        }
                    }
                    break;
            }

            if (Param.Target <= 0)
            {
                FLY.ControlLibrary.Window_WarningTip.Show("参数出错", "目标值<=0 异常");
                return;
            }

            if (Param.TolerancePercent <= 0)
            {
                FLY.ControlLibrary.Window_WarningTip.Show("参数出错", "公差 % <=0 异常");
                return;
            }

            Misc.PropertiesManager.CopyTo(Param,profileService.Param);
            profileService.Apply();

            if (!ProfileList.Contains(Param.PName))
            {
                ProfileList.Add(Param.PName);
            }
            FLY.ControlLibrary.Window_Tip.Show("应用 & 保存成功",
                Param.PName,
                TimeSpan.FromSeconds(2));

        }

        private void Del()
        {
            if (listview_profile.SelectedItem != null)
            {
                string productname = listview_profile.SelectedItem as string;
                profileService.Del(productname);
                ProfileList.Remove(productname);
                FLY.ControlLibrary.Window_Tip.Show("删除成功",
                    productname,
                    TimeSpan.FromSeconds(2));
            }
        }

        private void Ahelper()
        {

            WdAHelper w = new WdAHelper();

            w.Init(Param.K);
            w.Owner = FLY.ControlLibrary.COMMON.GetWindow(listview_profile);
            if (w.ShowDialog() == true)
            {
                Param.K = w.A;
            }
        }


        private void Legend()
        {
            WdProfileLegend w = new WdProfileLegend();
            w.DataContext = Param;
            w.Owner = FLY.ControlLibrary.COMMON.GetWindow(listview_profile);
            w.ShowDialog();
        }
    }
}
