﻿using FLY.ControlLibrary;
using FLY.Thick.Base.IService;
using FLY.Thick.BulkDataModule;
using MultiLayout;
using MultiLayout.UiModule;
using System;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using Unity;

namespace FLY.Thick.Blowing.UI.Fix.Client.UiModule
{
    /// <summary>
    /// ScanGraph.xaml 的交互逻辑
    /// </summary>
    public partial class ScanGraph : UserControl
    {
        ScanGraphItemParam graphparam;
        BulkDataServiceClientAdv bulkDataServiceClientAdv;
        IDynAreaService dynAreaService;
        public ScanGraph()
        {
            InitializeComponent();
        }

        [InjectionMethod]
        public void Init(int id,
            IDynAreaService dynAreaService,
            BulkDataServiceClientAdv bulkDataServiceClientAdv)
        {
            this.dynAreaService = dynAreaService;
            this.bulkDataServiceClientAdv = bulkDataServiceClientAdv;

            //查找参数
            graphparam = ScanGraphParams.Current.Items.Find(p => p.ID == id);
            if (graphparam == null)
            {
                graphparam = new ScanGraphItemParam() { ID = id };
                ScanGraphParams.Current.Items.Add(graphparam);
                ScanGraphParams.Current.Save();
            }

            FLY.ControlLibrary.GraphScan graph = graphscan;
            Misc.BindingOperations.SetBinding(graphparam, "ChartType", () =>
            {
                if (graphparam.ChartType == "Line")
                    graph.ChartType = System.Windows.Forms.DataVisualization.Charting.SeriesChartType.Line;
                else
                    graph.ChartType = System.Windows.Forms.DataVisualization.Charting.SeriesChartType.Column;
            });

            graph.DataContext = graphparam;
            //graph.SetValue(GraphScan.MarginProperty, new Thickness(5));
            graph.SetBinding(GraphScan.TitleProperty, new Binding("Title"));
            graph.SetBinding(Grid.ColumnProperty, new Binding("Column"));
            graph.SetBinding(Grid.RowProperty, new Binding("Row"));
            graph.SetBinding(Grid.ColumnSpanProperty, new Binding("ColumnSpan"));
            graph.SetBinding(Grid.RowSpanProperty, new Binding("RowSpan"));
            graph.SetBinding(GraphScan.IsAutoTargetProperty, new Binding("IsAutoTarget"));
            graph.SetBinding(GraphScan.IsPercentProperty, new Binding("IsPercent"));
            graph.SetBinding(GraphScan.YRangePercentProperty, new Binding("YRangePercent"));

            graph.SetBinding(GraphScan.IsReversedProperty, new Binding("IsReversed") { Source = ScanGraphParams.Current });
            graph.SetBinding(GraphScan.LeftRightIsVisableProperty, new Binding("LRIsVisable") { Source = ScanGraphParams.Current });
            graph.SetBinding(GraphScan.LeftRightIsReversedProperty, new Binding("LRIsReversed") { Source = ScanGraphParams.Current });


            //报警线
            //{
            //    graph.SetBinding(GraphScan.HasCtrlLineProperty, new Binding("Enable") { Source = gage.mScanWarningService });
            //    Binding b0 = new Binding("Alarm") { Source = dynAreaService.DynArea };

            //    Binding b1 = new Binding("AlarmLine") { Source = gage.mScanWarningService };
            //    {
            //        MultiBinding mb = new MultiBinding() { Mode = BindingMode.OneWay, Converter = new CtrlLineConverter() };
            //        mb.Bindings.Add(b0);
            //        mb.Bindings.Add(b1);
            //        graph.SetBinding(GraphScan.CtrlLineProperty, mb);
            //    }
            //}

            {
                graph.SetBinding(GraphScan.FirstBoltNoProperty, new Binding("FirstBoltNo") { Source = dynAreaService.DynArea });

                //graph.SetValue(GraphScan.FirstIndexProperty, 0);
                //graph.SetBinding(GraphScan.LastIndexProperty, new Binding("NBolts") { Source = gage.mDynArea });

                Binding b0 = new Binding("FirstBoltNo") { Source = dynAreaService.DynArea };
                Binding b1 = new Binding("DataBoltNoBegin") { Source = dynAreaService.DynArea };
                Binding b2 = new Binding("DataBoltNoEnd") { Source = dynAreaService.DynArea };
                Binding b3 = new Binding("ScanBoltNoBegin") { Source = dynAreaService.DynArea };
                Binding b4 = new Binding("ScanBoltNoEnd") { Source = dynAreaService.DynArea };

                {
                    MultiBinding mb = new MultiBinding() { Mode = BindingMode.OneWay, Converter = new FLY.Thick.Base.UI.Converter.BoltNo2BoltIndexConverter() };
                    mb.Bindings.Add(b0);
                    mb.Bindings.Add(b1);
                    graph.SetBinding(GraphScan.DataFirstProperty, mb);
                }

                {
                    MultiBinding mb = new MultiBinding() { Mode = BindingMode.OneWay, Converter = new FLY.Thick.Base.UI.Converter.BoltNo2BoltIndexConverter() };
                    mb.Bindings.Add(b0);
                    mb.Bindings.Add(b2);
                    graph.SetBinding(GraphScan.DataLastProperty, mb);
                }

                {
                    MultiBinding mb = new MultiBinding() { Mode = BindingMode.OneWay, Converter = new FLY.Thick.Base.UI.Converter.BoltNo2BoltIndexConverter() };
                    mb.Bindings.Add(b0);
                    mb.Bindings.Add(b3);
                    graph.SetBinding(GraphScan.FirstIndexProperty, mb);
                }

                {
                    MultiBinding mb = new MultiBinding() { Mode = BindingMode.OneWay, Converter = new FLY.Thick.Base.UI.Converter.BoltNo2BoltIndexConverter() };
                    mb.Bindings.Add(b0);
                    mb.Bindings.Add(b4);
                    graph.SetBinding(GraphScan.LastIndexProperty, mb);
                }
            }

            graph.SetBinding(GraphScan.TargetProperty, new Binding("Target") { Source = dynAreaService.DynArea });
            graph.SetBinding(GraphScan.AlarmProperty, new Binding("Alarm") { Source = dynAreaService.DynArea });


            //bulkdata 数据绑定！！！
            BulkDataServiceClientAdv.BulkDataGraphFrame bulkdata_client = new BulkDataServiceClientAdv.BulkDataGraphFrame();
            //graphparam 绑定到 bulkdataclient
            Misc.BindingOperations.SetBinding(graphparam, "BM", bulkdata_client, "BM");
            Misc.BindingOperations.SetBinding(graphparam, "Mix", bulkdata_client, "Mix");


            graph.SetBinding(GraphScan.Title2Property, new Binding("Info") { Source = bulkdata_client });
            graph.SetBinding(GraphScan.IsValidProperty, new Binding("Type") { Source = bulkdata_client, Converter = new IsValidConverter() });
            graph.DataSource = bulkdata_client.Frame;


            graph.ShowSettingDialogEvent += new EventHandler(delegate (object sender, EventArgs e)
            {
                ScanGraphConfig window = new ScanGraphConfig();
                window.Init((sender as GraphScan).DataContext as ScanGraphItemParam);
                window.Owner = FLY.ControlLibrary.COMMON.GetWindow(graph);
                window.ShowDialog();
            });

            //BulkDataServiceClientAdv mBulkDataClient;
            //if(mBulkDataClients.ContainsKey(graphparam.BulkData))
            //    mBulkDataClient = mBulkDataClients[graphparam.BulkData];
            //else
            //    mBulkDataClient = mBulkDataClients.First().Value;
            bulkDataServiceClientAdv.Add(bulkdata_client);
        }
    }

    /// <summary>
    /// 扫描图的参数
    /// </summary>
    public class ScanGraphItemParam : UIModuleParam
    {
        public string UiModule { get; set; } = "scangraph";
        /// <summary>
        /// 曲线类型
        /// </summary>
        public string ChartType { get; set; } = "Column";

        /// <summary>
        /// 记录点
        /// </summary>
        public int BM { get; set; } = 0;

        /// <summary>
        /// Y轴比例
        /// </summary>
        public double YRangePercent { get; set; } = 3;

        /// <summary>
        /// 标题
        /// </summary>
        public string Title { get; set; } = "扫描图";


        /// <summary>
        /// 混合图
        /// </summary>
        public int Mix { get; set; } = 1;

        /// <summary>
        /// 自动目标值
        /// </summary>
        public bool IsAutoTarget { get; set; } = false;


        /// <summary>
        /// % 显示
        /// </summary>
        public bool IsPercent { get; set; } = false;
    }

    /// <summary>
    /// 全部扫描图的参数
    /// </summary>
    public class ScanGraphParams : UIModuleParams<ScanGraphItemParam>
    {

        /// <summary>
        /// 主界面反向 
        /// </summary>
        public bool IsReversed { get; set; }

        /// <summary>
        /// 显示 左右标示
        /// </summary>
        public bool LRIsVisable { get; set; } = false;


        /// <summary>
        /// 左右标示 反向
        /// </summary>
        public bool LRIsReversed { get; set; } = false;


        /// <summary>
        /// 扫描图轴间隔 ,单位分区
        /// </summary>
        public int XInterval { get; set; } = 10;


        static ScanGraphParams()
        {
        }
        public ScanGraphParams()
        {
            string path = System.IO.Path.Combine(FlyLayout.BasePath, "scangraph.json");
            FilePath = path;
        }

        static ScanGraphParams current = null;
        public static ScanGraphParams Current
        {
            get
            {
                if (current == null)
                {
                    current = new ScanGraphParams();
                    current.Load();
                }
                return current;
            }
        }

        public void MatchParam(string uiModule, int[] IDs)
        {
            if (Items.RemoveAll(i => { return (i.UiModule == uiModule) && (!IDs.Contains(i.ID)); }) > 0)
            {
                Save();
            }
        }
        public override void MatchParam(int[] IDs)
        {
            MatchParam("scangraph", IDs);
        }

    }

    /// <summary>
    /// 扫描图控件模块
    /// </summary>
    public class UiModule2_ScanGraph : MultiLayout.UiModule.IUiModule2
    {

        /// <summary>
        /// 控件标题
        /// 它的值取决于culture
        /// </summary>
        public string Title => "扫描图";
        public ComponentType Type => ComponentType.Graph;
        public bool IsUnique => false;

        /// <summary>
        /// 控件
        /// 创建时，需要给它唯一ID，让加载自己的数据
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public FrameworkElement GetComponent(int id, IUnityContainer container)
        {
            return container.Resolve<ScanGraph>(new Unity.Resolution.ParameterOverride("id", id));
        }

        /// <summary>
        /// 控件缩略图，用于编辑界面时，大致看看
        /// 创建时，需要给它唯一ID，让加载自己的数据
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public FrameworkElement GetThumbnail()
        {
            return new Grid();
        }

        /// <summary>
        /// 给出全部控件ID, 控件自行删除没有的参数
        /// </summary>
        /// <param name="IDs"></param>
        public void MatchParam(int[] IDs)
        {
            ScanGraphParams.Current.MatchParam(IDs);
        }
    }


    public class IsValidConverter : IValueConverter
    {

        #region IValueConverter 成员

        public object Convert(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {

            FR_DATA_TYPE type = (FR_DATA_TYPE)value;
            if (type == FR_DATA_TYPE.VALID)
                return true;
            else
                return false;

        }

        public object ConvertBack(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            throw new NotImplementedException();
        }

        #endregion

    }
    public class CtrlLineConverter : IMultiValueConverter
    {


        /// <summary>
        /// values[0] = int alarm, values[1] = double alarmline
        /// </summary>
        /// <param name="values"></param>
        /// <param name="targetType"></param>
        /// <param name="parameter"></param>
        /// <param name="culture"></param>
        /// <returns></returns>
        public object Convert(object[] values, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            if (values.Count() != 2)
                return null;

            if ((!(values[0] is int)) || (!(values[1] is double)))
                return null;

            int alarm = (int)values[0];
            double alarmline = (double)values[1];
            return (int)(alarm * alarmline);
        }

        public object[] ConvertBack(object value, Type[] targetTypes, object parameter, System.Globalization.CultureInfo culture)
        {
            throw new NotImplementedException();
        }
    }
}
