﻿using GalaSoft.MvvmLight.Command;
using LiveCharts;
using LiveCharts.Wpf;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using C1.WPF.C1Chart;

namespace FLY.HeatingHelper.UI.UiModule
{
    /// <summary>
    /// UC_FramePicker.xaml 的交互逻辑
    /// </summary>
    public partial class UC_FramePicker : UserControl
    {
        public UC_FramePicker()
        {
            InitializeComponent();
        }

        private void DataSeries_PlotElementLoaded(object sender, EventArgs e)
        {
            (DataContext as UC_FramePickerViewModel).PELoaded(sender, e);
        }
    }

    public class UC_FramePickerViewModel : INotifyPropertyChanged
    {
        public event PropertyChangedEventHandler PropertyChanged;

        public UC_FramePickerViewModel()
        {
            InitCmd();
        }

        //ThickHeatData4SQLite_V1 _data;
        IThickHeatData _data;

        private int _selectedFrame;
        public int SelectedFrame
        {
            get
            {
                return _selectedFrame;
            }
            set
            {
                if (_selectedFrame != value)
                {
                    _selectedFrame = value;
                    UpdateData();
                }
            }
        }

        private int _minFrame;
        public int MinFrame
        {
            get { return _minFrame; }
            set
            {
                if (_minFrame != value)
                {
                    if (SelectedFrame < value)
                    {
                        SelectedFrame = value;
                    }
                    _minFrame = value;
                }
            }
        }

        private int _maxFrame;
        public int MaxFrame
        {
            get
            {
                return _maxFrame;
            }
            set
            {
                if (_maxFrame != value)
                {
                    if (SelectedFrame > value)
                    {
                        SelectedFrame = value;
                    }
                    _maxFrame = value;
                }
            }
        }

        private int _resetBolt;
        public int ResetBolt
        {
            get
            {
                return _resetBolt;
            }
            set
            {
                if (_resetBolt != value)
                {
                    _resetBolt = value;
                }
            }
        }

        private double _rotAngle = double.NaN;
        public double RotAngle
        {
            get
            {
                return _rotAngle;
            }
            set
            {
                if ((Math.Abs(_rotAngle - value) >= 0.00099) || double.IsNaN(_rotAngle))
                {
                    _rotAngle = value;
                }
            }
        }

        public int FrameID
        {
            get => _data == null ? -2 : _data.Index2ID(SelectedFrame);
        }

        public DateTime FrameTime
        {
            get
            {
                if (_data == null) return DateTime.MinValue;
                if (SelectedFrame < MinFrame || SelectedFrame > MaxFrame) return DateTime.MinValue;
                if (SelectedFrame < 0 || SelectedFrame >= _data.Dat_Times.Count()) return DateTime.MinValue;
                return _data.Dat_Times[SelectedFrame];
            }
        }
        public RelayCommand PreCmd { get; internal set; }
        public RelayCommand NxtCmd { get; internal set; }

        public ChartData myData { get; set; } = new ChartData();

        /// <summary>
        /// 初始化数据
        /// </summary>
        /// <param name="_dat"></param>
        public void InitData(object _dat, int from, int to)
        {
            _data = _dat as IThickHeatData; /*ThickHeatData4SQLite_V1*/
            int update = 2;
            if (from > 0)
            {
                MinFrame = from;
                update--;
            }
            if ((to > from) && (to > 0)) 
            {
                MaxFrame = to;
                SelectedFrame = to;
                update--;
            }
            if (update > 0)
            {
                //UpdateData();
            }
        }

        public void InitData(IThickHeatData _dat)
        {
            _data = _dat as IThickHeatData; /*ThickHeatData4SQLite_V1*/
            MinFrame = _data.AnalystIndexFrom;
            MaxFrame = _data.AnalystIndexTo;
            //UpdateData();
        }

        /// <summary>
        /// 界面加载时调用
        /// </summary>
        public void OnLoaded(bool selectMin=false)
        {
            MinFrame = _data.AnalystIndexFrom;
            MaxFrame = _data.AnalystIndexTo;
            SelectedFrame = selectMin ? MinFrame : MaxFrame;
            UpdateData();
        }

        //private ChartValues<double> v1 = new ChartValues<double>();
        //private ChartValues<double> v2 = new ChartValues<double>();
        #region For C1Chart
        public DoubleCollection DS_thick { get; set; }

        public DoubleCollection DS_heat { get; set; }

        public double Thick_Mean { get; set; }

        public double AsisY_Min { get; set; }

        public double AsisY_Max { get; set; }

        public List<KeyValuePair<object, double>> AsisY_Ticks { get; set; }

        public void PELoaded(object s, EventArgs e)
        {
            PlotElement pe = s as PlotElement;
            double v = Math.Abs(pe.DataPoint.Value);
            if(v <=AsisY_Max / 3)
            {
                pe.Fill = new SolidColorBrush(Colors.Green);
            }
            else if (v <= AsisY_Max * 2 / 3)
            {
                pe.Fill = new SolidColorBrush(Colors.Yellow);
            }
            else
            {
                pe.Fill = new SolidColorBrush(Colors.Red);
            }
        }
        #endregion
        private void UpdateData()
        {
            if (SelectedFrame < 0) return;
            ResetBolt = _data.GetResetBoltByIndex(SelectedFrame);
            RotAngle = _data.GetRotAngleByIndex(SelectedFrame);
            Thick_Mean = _data.Dat_Means[SelectedFrame];
            double[] tmp = _data.GetThicksByIndex(SelectedFrame, ResetBolt, RotAngle);
            double[] tmph = _data.GetHeatsByIndex(SelectedFrame, -1, -1);
            if (tmp == null || tmph == null) return;
            for(int i=0;i<tmp.Count();i++)
            {
                if (double.IsNaN(tmp[i])) continue;
                tmp[i] -= Thick_Mean;
            }
            AsisY_Min = - Thick_Mean * 0.09;
            AsisY_Max = Thick_Mean * 0.09;
            AsisY_Ticks = new List<KeyValuePair<object, double>>
            {
                new KeyValuePair<object, double>(Thick_Mean.ToString("F1"), 0),
                new KeyValuePair<object, double>("3%", Thick_Mean * 0.03),
                new KeyValuePair<object, double>("-3%", -Thick_Mean * 0.03),
                new KeyValuePair<object, double>("6%", Thick_Mean * 0.06),
                new KeyValuePair<object, double>("-6%", -Thick_Mean * 0.06),
                new KeyValuePair<object, double>("9%", Thick_Mean * 0.09),
                new KeyValuePair<object, double>("-9%", -Thick_Mean * 0.09)
            };
            DS_thick = new DoubleCollection(tmp);
            DS_heat = new DoubleCollection(tmph);
        }

        public void InitCmd()
        {
            PreCmd = new RelayCommand(() =>
            {
                if (SelectedFrame > MinFrame)
                {
                    SelectedFrame--;
                }
            });
            NxtCmd = new RelayCommand(() =>
            {
                if (SelectedFrame < MaxFrame)
                {
                    SelectedFrame++;
                }
            });
        }
    }
}
