﻿using C1.WPF.C1Chart;
using GalaSoft.MvvmLight.Command;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.ServiceModel.Channels;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Media;

namespace FLY.HeatingHelper.UI.UiModule.UC_SelectData
{
    class ViewModel_UC_SelectData : INotifyPropertyChanged
    {
        public ViewModel_UC_SelectData(IThickHeatData dat)
        {
            Dat = dat;
            DefaultLoadDataRange = new RelayCommand(Command_DefaultLoadDataRange);
            ReloadData = new RelayCommand(Command_ReloadData);
            PreData = new RelayCommand(Command_PreData);
            NextData = new RelayCommand(Command_NextData);
            DefaultAnalystData = new RelayCommand(Command_DefaultAnalystData);
            Test = new RelayCommand(Command_Test);
            Command_DefaultLoadDataRange();
            Misc.BindingOperations.SetBinding(Dat, "AnalystIndexFrom", () =>
            {
                
                AnalystDataFrom = null;
            });
            Misc.BindingOperations.SetBinding(Dat, "AnalystIndexTo", () =>
            {
                AnalystDataTo = null;
            });
        }

        #region UI界面
        public IThickHeatData Dat { get; set; } = null;

        public DateTime? TotalDataFrom { get; set; } = null;

        public int RecordCount
        {
            get 
            {
                if (Dat is null) return -1;
                return Dat.Dat_Times.Count();
            }
            set
            {

            }
        }

        public double[] DS_thick { get; set; }
        public double[] DS_2sigma { get; set; }
        public double[] DS_2sigmaN { get; set; }

        public DateTime? AnalystDataFrom
        {
            get
            {
                if (Dat is null) return null;
                if (Dat.AnalystIndexFrom >= 0)
                {
                    return Dat.Dat_Times[Dat.AnalystIndexFrom];
                }
                return null;
            }
            set
            {

            }
        }
        public DateTime? AnalystDataTo
        {
            get
            {
                if (Dat is null) return null;
                if (Dat.AnalystIndexTo >= 0)
                {
                    return Dat.Dat_Times[Dat.AnalystIndexTo];
                }
                return null;
            }
            set
            {

            }
        }
        #endregion

        #region 内部功能函数
        private void UpdateChart()
        {
            if (RecordCount <= 0) return;
            DS_thick = Dat.Dat_Means.ToArray();
            double[] tmp0 = Dat.Dat_Sigmas.ToArray();
            int cnt = tmp0.Count();
            double[] tmp = new double[cnt];
            for (int i = 0; i < cnt; i++)
            {
                if (double.IsNaN(DS_thick[i]))
                {
                    tmp[i] = tmp0[i] = double.NaN;
                }
                if (Math.Abs(DS_thick[i]) < 1)
                {
                    tmp[i] = tmp0[i] = double.NaN;
                }
                tmp0[i] = tmp0[i] / DS_thick[i];
                tmp[i] = -tmp0[i];
            }
            DS_2sigma = tmp0;
            DS_2sigmaN = tmp;
        }
        #endregion

        public void PlotElementLoaded(object s, EventArgs e)
        {
            PlotElement pe = s as PlotElement;
            double v = Math.Abs(pe.DataPoint.Value);
            if (v <= 0.03)
            {
                pe.Fill = new SolidColorBrush(Colors.Green);
            }
            else if (v <= 0.06)
            {
                pe.Fill = new SolidColorBrush(Colors.Yellow);
            }
            else
            {
                pe.Fill = new SolidColorBrush(Colors.Red);
            }
        }

        #region UI界面按钮功能(Command)
        public RelayCommand DefaultLoadDataRange { get; internal set; } = null;

        private void Command_DefaultLoadDataRange()
        {
            Dat.SetLoadDaTaDefault();
        }

        public RelayCommand ReloadData { get; internal set; } = null;

        private void Command_ReloadData()
        {
            if (Dat is null) return;
            Dat.LoadDataFromDB(1);
            Dat.ResetCluster();
            Dat.ClusterOnThickMean();
            RecordCount = 1;
            OnLoaded();
            //UpdateChart();
        }

        public RelayCommand PreData { get; internal set; } = null;

        private void Command_PreData()
        {
            int from = 0, to = 0;
            if (Dat.CalculateFromToByClass(int.MaxValue, ref from, ref to) < 0) return;
        }

        public RelayCommand NextData { get; internal set; } = null;

        private void Command_NextData()
        {
            int from = 0, to = 0;
            if (Dat.CalculateFromToByClass(int.MinValue, ref from, ref to) < 0) return;
        }

        public RelayCommand DefaultAnalystData { get; internal set; } = null;

        private void Command_DefaultAnalystData()
        {
            Dat.SetDefaultAnalystIndexs();
        }

        public RelayCommand Test { get; set; } = null;
        
        private void Command_Test()
        {
        }

        public void OnLoaded()
        {
            Command_DefaultAnalystData();
            UpdateChart();
        }

        #endregion
        public event PropertyChangedEventHandler PropertyChanged;
    }
}
