﻿using FLY.OBJComponents.Client;
using FLY.Weight.Client;
using FLY.Weight.Common;
using FLY.Weight.IService;
using MultiLayout;
using MultiLayout.UiModule;
using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Unity;

namespace FLY.Weight.UI.Client.UiModule
{
    /// <summary>
    /// Page_Main.xaml 的交互逻辑
    /// </summary>
    public partial class MainGraph : UserControl
    {
        IUnityContainer container;
        IWeightSystemService weightSystemService;

        SetPLCUpdatePlan setPlan_accessory;
        List<SetPLCUpdatePlan> setPlan_items = new List<SetPLCUpdatePlan>();
        public MainGraph()
        {
            InitializeComponent();

        }

        [InjectionMethod]
        public void Init(
            IUnityContainer container,
            IWeightSystemService weightSystemService)
        {
            this.container = container;
            this.weightSystemService = weightSystemService;
            root_grid.DataContext = this.weightSystemService;

            setPlan_accessory = new SetPLCUpdatePlan(
                this.weightSystemService.PLCos,
                this.weightSystemService.Accessory,
                new string[] {
                        "TotalFlowSetting",
                        "TotalFlow",
                        "TotalProduction",

                        "ALast",
                        "ACurrent",
                        "ACurrentLen",
                        "AClear",

                        "BLast",
                        "BCurrent",
                        "BCurrentLen",
                        "BClear",

                        "TotalFilmWidth",
                        "IsRimNoRecycle",
                        "RimWidth",
                        "ActFilmWidth",
                        "SetThickness",
                        "TargetVelocity",
                        "CurrentVelocity"
                });

            for (int i = 0; i < this.weightSystemService.Items.Count(); i++)
            {
                SetPLCUpdatePlan plan = new SetPLCUpdatePlan(
                    this.weightSystemService.PLCos,
                    this.weightSystemService.Items[i],
                    item_update_propertynames);
                setPlan_items.Add(plan);
            }
            if (weightSystemService is WeightSystemServiceClient) { 

                (weightSystemService as WeightSystemServiceClient).ResetItemsEvent += MWeighterCsService_ResetItemsEvent;
            }
        }

        private void MWeighterCsService_ResetItemsEvent()
        {
            //把多出来的删除
            int remove_cnt = setPlan_items.Count() - weightSystemService.Items.Count();
            if (remove_cnt > 0)
            {
                for (int i = 0; i < remove_cnt; i++)
                {
                    SetPLCUpdatePlan plan = setPlan_items[setPlan_items.Count() - 1 - i];
                    plan.Dispose();
                }
                setPlan_items.RemoveRange(setPlan_items.Count() - remove_cnt, remove_cnt);
            }
            else
            {
                int start_idx = setPlan_items.Count();
                int add_cnt = -remove_cnt;
                for (int i = 0; i < add_cnt; i++)
                {
                    SetPLCUpdatePlan plan = new SetPLCUpdatePlan(
                    weightSystemService.PLCos,
                    weightSystemService.Items[start_idx+i],
                    item_update_propertynames);
                    setPlan_items.Add(plan);
                }
            }
        }
        string[] item_update_propertynames = new string[] {
            "CurrentFlow",
            "CumulativeProduction",
            "FlowSetting",
            "CurrentFlow",
            "ScrewPDisp",
            "ScrewPSet",
            "ScrewPDisp",
            "MixBucketWeight",
            "BinWeight",
            "MixIsOn",
            "BucketValveIsOpen",
            "ScrewMotorFreq",
            "ScrewMotorIsOn",
            "ScrewIsAutoMode",
            "ScrewManualFreq"
        };

        private async void button_frequency_Click(object sender, RoutedEventArgs e)
        {
            WdFrequency w = new WdFrequency();
            var weight = ((Button)sender).Tag as FLY.Weight.Common.WeighterC;
            w.FrequencySet = weight.ScrewManualFreq;
            w.Owner = FLY.ControlLibrary.COMMON.GetWindow(this);
            if (w.ShowDialog()==true)
            {
                //TODO
                weight.ScrewManualFreq = (float)w.FrequencySet;
                weight.ScrewManualFreqIsSet = true;
                //不用写下降沿！！！！！
            }
        }

        private void button_ratio_Click(object sender, RoutedEventArgs e)
        {
            WdRatioSet w = new WdRatioSet();
            w.Owner = FLY.ControlLibrary.COMMON.GetWindow(this);
            w.Init(weightSystemService);
            w.ShowDialog();
        }

        private void button_info_Click(object sender, RoutedEventArgs e)
        {
            PgMenu p = new PgMenu();
            container.BuildUp(p);
            MultiLayout.FlyLayoutManager.NavigationService.Navigate(p);
        }

        private void button_clear_Click(object sender, RoutedEventArgs e)
        {
            foreach (FLY.Weight.Common.WeighterC weight in weightSystemService.Items)
            {
                weight.ClearProduction = true;
            }
            FLY.ControlLibrary.Window_Tip.Show("", "清空成功", TimeSpan.FromSeconds(2));
        }

        private void button_ingredient_Click(object sender, RoutedEventArgs e)
        {
            WdIngredient w = new WdIngredient();
            var weight = ((Button)sender).Tag as WeighterC;

            w.Init(container, weight, weightSystemService);


            w.Owner = FLY.ControlLibrary.COMMON.GetWindow(this);
            w.ShowDialog();
        }

        private void button_thickness_Click(object sender, RoutedEventArgs e)
        {
            WdThickness w = new WdThickness();
            if (weightSystemService.Items.Count() < 1)
                return;

            w.SetThickness = (int)weightSystemService.Accessory.SetThickness;
            w.FilmWidth = (int)weightSystemService.Accessory.TotalFilmWidth;
            w.RimWidth = (int)weightSystemService.Accessory.RimWidth;
            w.IsRimNoRecycle = weightSystemService.Accessory.IsRimNoRecycle;

            

            w.Owner = FLY.ControlLibrary.COMMON.GetWindow(this);

            if (w.ShowDialog() == true)
            {
                weightSystemService.Accessory.SetThickness = w.SetThickness;
                weightSystemService.Accessory.TotalFilmWidth = w.FilmWidth;
                weightSystemService.Accessory.RimWidth = w.RimWidth;
                weightSystemService.Accessory.IsRimNoRecycle = w.IsRimNoRecycle;

                FLY.ControlLibrary.Window_Tip.Show("","厚度参数设置成功", TimeSpan.FromSeconds(2));
            }
        }

        private void button_aclear_Click(object sender, RoutedEventArgs e)
        {
            if (weightSystemService.ItemsCnt > 0)
            {
                weightSystemService.Accessory.AClear = true;
                FLY.ControlLibrary.Window_Tip.Show("", "清空成功", TimeSpan.FromSeconds(2));
            }
        }

        private void button_bclear_Click(object sender, RoutedEventArgs e)
        {
            if (weightSystemService.ItemsCnt > 0)
            {
                weightSystemService.Accessory.BClear = true;
                FLY.ControlLibrary.Window_Tip.Show("", "清空成功", TimeSpan.FromSeconds(2));
            }
        }
    }




    public class UiModule2_MainGraph : IUiModule2
    {
        /// <summary>
        /// 控件标题
        /// 它的值取决于culture
        /// </summary>
        public string Title => "称重主界面";

        public ComponentType Type => ComponentType.Graph;

        public bool IsUnique => true;

        /// <summary>
        /// 控件
        /// 创建时，需要给它唯一ID，让加载自己的数据
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public FrameworkElement GetComponent(int id, IUnityContainer container)
        {
            MainGraph graph = new MainGraph();
            container.BuildUp(graph);
            return graph;
        }

        /// <summary>
        /// 控件缩略图，用于编辑界面时，大致看看
        /// 创建时，需要给它唯一ID，让加载自己的数据
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public FrameworkElement GetThumbnail()
        {
            return new System.Windows.Controls.Grid();
        }


        /// <summary>
        /// 给出全部控件ID, 控件自行删除没有的参数
        /// </summary>
        /// <param name="IDs"></param>
        public void MatchParam(int[] IDs)
        {
        }
    }
}
