﻿using FLY.OBJComponents.Client;
using FLY.Weight.Client;
using FLY.Weight.IService;
using MultiLayout.UiModule;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Unity;

namespace FLY.Weight.UI.Client.UiModule
{
    /// <summary>
    /// UcWeighterItem.xaml 的交互逻辑
    /// </summary>
    public partial class UcWeighterItem : UserControl
    {
        IUnityContainer container;
        IWeightSystemService weightSystemService;
        List<SetPLCUpdatePlan> setPlan_items = new List<SetPLCUpdatePlan>();
        int Index = 0;

        public UcWeighterItem()
        {
            InitializeComponent();
        }

        [InjectionMethod]
        public void Init(
            IUnityContainer container,
            int id, 
            IWeightSystemService weightSystemService)
        {
            this.container = container;
            this.weightSystemService = weightSystemService;

            //查找参数
            if (WeighterItemParams.Current.Indexs.ContainsKey(id))
            {
                Index = WeighterItemParams.Current.Indexs[id];
            }
            else 
            {
                for (int i = 0; i < this.weightSystemService.Items.Count(); i++) 
                {
                    if (!WeighterItemParams.Current.Indexs.ContainsValue(i)) 
                    {
                        WeighterItemParams.Current.Indexs.Add(id, i);
                        Index = i;
                        break;
                    }
                }
            }

            int idx = Index;
            if (idx >= this.weightSystemService.Items.Count())
                idx = this.weightSystemService.Items.Count() - 1;
            else if (idx < 0)
                idx = 0;

            this.DataContext = this.weightSystemService.Items[idx];

            for (int i = 0; i < this.weightSystemService.Items.Count(); i++)
            {
                SetPLCUpdatePlan plan = new SetPLCUpdatePlan(
                    this.weightSystemService.PLCos,
                    this.weightSystemService.Items[i],
                    item_update_propertynames);
                setPlan_items.Add(plan);
            }

            if (weightSystemService is WeightSystemServiceClient)
                (weightSystemService as WeightSystemServiceClient).ResetItemsEvent += MWeighterCsService_ResetItemsEvent;
        }

        private void MWeighterCsService_ResetItemsEvent()
        {
            //把多出来的删除
            int remove_cnt = setPlan_items.Count() - weightSystemService.Items.Count();
            if (remove_cnt > 0)
            {
                for (int i = 0; i < remove_cnt; i++)
                {
                    SetPLCUpdatePlan plan = setPlan_items[setPlan_items.Count() - 1 - i];
                    plan.Dispose();
                }
                setPlan_items.RemoveRange(setPlan_items.Count() - remove_cnt, remove_cnt);
            }
            else
            {
                int start_idx = setPlan_items.Count();
                int add_cnt = -remove_cnt;
                for (int i = 0; i < add_cnt; i++)
                {
                    SetPLCUpdatePlan plan = new SetPLCUpdatePlan(
                    weightSystemService.PLCos,
                    weightSystemService.Items[start_idx + i],
                    item_update_propertynames);
                    setPlan_items.Add(plan);
                }
            }
        }
        string[] item_update_propertynames = new string[] {
           // "CumulativeProduction",
            "FlowSetting",//√
            "CurrentFlow",//√
            "ScrewPDisp",//√
            "ScrewPSet",//√
            "ScrewPDisp",//√
            "MixBucketWeight",//√
            "BinWeight",//√
            "MixIsOn",//√
            "BucketValveIsOpen",//√
            "ScrewMotorFreq",//√
            "ScrewMotorIsOn",//√
            "ScrewIsAutoMode",//√
            "ScrewManualFreq"//√
        };

        private async void button_frequency_Click(object sender, RoutedEventArgs e)
        {
            WdFrequency w = new WdFrequency();
            var weight = ((Button)sender).Tag as FLY.Weight.Common.WeighterC;
            w.FrequencySet = weight.ScrewManualFreq;
            w.Owner = FLY.ControlLibrary.COMMON.GetWindow(this);
            if (w.ShowDialog() == true)
            {
                //TODO
                weight.ScrewManualFreq = (float)w.FrequencySet;
                weight.ScrewManualFreqIsSet = true;
                //不用写下降沿！！！！！
            }
        }

        private void button_ratio_Click(object sender, RoutedEventArgs e)
        {
            WdRatioSet w = new WdRatioSet();
            w.Owner = FLY.ControlLibrary.COMMON.GetWindow(this);
            w.Init(weightSystemService);
            w.ShowDialog();
        }

        private void button_ingredient_Click(object sender, RoutedEventArgs e)
        {
            WdIngredient w = new WdIngredient();
            FLY.Weight.Common.WeighterC weight = ((Button)sender).Tag as FLY.Weight.Common.WeighterC;
            w.Init(container, weight, weightSystemService);

            w.Owner = FLY.ControlLibrary.COMMON.GetWindow(this);
            w.ShowDialog();
        }
    }

    /// <summary>
    /// 全部扫描图的参数
    /// </summary>
    public class WeighterItemParams 
    {
        public Dictionary<int, int> Indexs = new Dictionary<int, int>();
        public WeighterItemParams()
        {
        }
        public static WeighterItemParams Current { get; } = new WeighterItemParams();
    }

    public class UiModule2_UcWeighterItem : MultiLayout.UiModule.IUiModule2
    {
        /// <summary>
        /// 控件标题
        /// 它的值取决于culture
        /// </summary>
        public string Title => "称重层控制";

        public ComponentType Type => ComponentType.Graph;

        public bool IsUnique => true;

        /// <summary>
        /// 控件
        /// 创建时，需要给它唯一ID，让加载自己的数据
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public FrameworkElement GetComponent(int id, IUnityContainer container)
        {
            UcWeighterItem graph = new UcWeighterItem();
            container.BuildUp(graph);
            return graph;
        }

        /// <summary>
        /// 控件缩略图，用于编辑界面时，大致看看
        /// 创建时，需要给它唯一ID，让加载自己的数据
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public FrameworkElement GetThumbnail()
        {
            return new System.Windows.Controls.Grid();
        }

        /// <summary>
        /// 给出全部控件ID, 控件自行删除没有的参数
        /// </summary>
        /// <param name="IDs"></param>
        public void MatchParam(int[] IDs)
        {
        }
    }
}
