﻿using OfficeOpenXml;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;

namespace Autogen_WS
{
    public class DevMapperRule
    {
        /// <summary>
        /// PLC寄存器类型
        /// </summary>
        public string PLCVarType;
  
        /// <summary>
        /// MB寄存器类型 [0区，1区，3区，4区]
        /// </summary>
        public string VarType;
        /// <summary>
        /// MB寄存器首地址
        /// </summary>
        public int Addr1st;

        /// <summary>
        /// N进制
        /// </summary>
        public int FromBase;
    }

    /// <summary>
    /// 设备转换表
    /// </summary>
    public class DevMapperRuleTable
    {
        public List<DevMapperRule> Tables = new List<DevMapperRule>();

        /// <summary>
        /// 读取 地址段映射
        /// </summary>
        /// <returns></returns>
        public bool Load(string xlsxfile)
        {
            Tables.Clear();

            //列序号 
            int i_plcVarType = 1;//PLC寄存器类型
            int i_vartype = 2;//MB寄存器类型
            int i_addr1st = 3;//MB寄存器首地址
            int i_fromBase = 4;//进制


            //Regex r_variablename = new Regex(@"设备[0-9]_(读写|只读)(\w+)");
            Regex r_mode = new Regex(@"\[(0|1|3|4)区\]");

            using (ExcelPackage packages = new ExcelPackage(new FileInfo(xlsxfile)))
            {
                using (ExcelWorksheet sheet = packages.Workbook.Worksheets["地址段映射"])
                {
                    int i = 1;
                    while (true)
                    {
                        i++;
                        DevMapperRule cell = new DevMapperRule();
                        string str_plcvt;//PLC寄存器类型
                        string str_vt;//MB寄存器类型
                        string str_a;//MB寄存器首地址
                        string str_base;
                        try
                        {
                            str_plcvt = sheet.Cells[i, i_plcVarType].Value.ToString();
                            str_vt = sheet.Cells[i, i_vartype].Value.ToString();
                            str_a = sheet.Cells[i, i_addr1st].Value.ToString();
                            str_base = sheet.Cells[i, i_fromBase].Value.ToString();
                        }
                        catch
                        {
                            //完成，没有数据了
                            break;
                        }
                        if (string.IsNullOrEmpty(str_plcvt)) break;


                        int num;


                        cell.PLCVarType = str_plcvt;

                        Match m = r_mode.Match(str_vt);
                        if (!m.Success)
                            return false;

                        cell.VarType = m.Groups[1].Value;

                        Regex regex = new Regex(@"0x([0-9a-fA-F]+)");
                        var match= regex.Match(str_a);

                        if (match.Success)
                        {
                            str_a = match.Groups[1].Value;
                            //16进制
                            if (!int.TryParse(str_a, System.Globalization.NumberStyles.HexNumber, null, out num))
                                return false;
                        }
                        else
                        {
                            if (!int.TryParse(str_a, out num))
                                return false;
                        }
                        cell.Addr1st = num;

                        if (!int.TryParse(str_base, out num))
                            return false;

                        cell.FromBase = num;

                        Tables.Add(cell);

                    }
                }
            }
            if (Tables.Count == 0)
                return false;
            return true;
        }
    }
}
